# These two functions will draw histograms with superimposed densities from a given model
plot_fit_errors_n <- function(ret, rv) {
	the_fit <- fit_rGARCH_GAS_n(ret, rv)
	fit <- recursive_filter_n(the_fit[["optim_vals"]], ret, log(rv))
	par(mfrow=c(2,1))
	hist(fit[["z"]], prob = T, breaks = 100, xlab = "z", main = "")
	curve(dnorm(x), add = T, col="red")
	hist(fit[["u"]], prob = T, breaks = 100, xlab = "u", main = "")
	curve(dnorm(x), add = T, col="red")
}

plot_fit_errors_t <- function(ret, rv) {
	the_fit <- fit_rGARCH_GAS_t(ret, rv)
	fit <- recursive_filter_t(the_fit[["optim_vals"]], ret, log(rv))
	par(mfrow=c(2,1))
	hist(fit[["z"]], prob = T, breaks = 100, xlab = "z", main = "")
	curve(dt(x, the_fit[["optim_vals"]][11]), add = T, col="red")
	hist(fit[["u"]], prob = T, breaks = 100, xlab = "u", main = "")
	curve(dt(x, the_fit[["optim_vals"]][12]), add = T, col="red")
}

# These two functions plot the fitted latent volatility against the realised counterpart
plot_fit_volatility_t <- function(ret, rv) {
	the_fit <- fit_rGARCH_GAS_t(ret, rv)
	fit <- recursive_filter_t(the_fit[["optim_vals"]], ret, log(rv))
	plot(fit[["lambda"]], type="l", col="red", ylab = "log(volatility)")
	points(log(rv), pch=20, col="gray")
}

plot_fit_volatility_n <- function(ret, rv) {
	the_fit <- fit_rGARCH_GAS_n(ret, rv)
	fit <- recursive_filter_n(the_fit[["optim_vals"]], ret, log(rv))
	plot(fit[["lambda"]], type="l", col="red", ylab = "log(volatility)")
	points(log(rv), pch=20, col="gray")
}

zip <- function(vector1, vector2) {
	if (length(vector1)==length(vector2)) {
		vals <- c()
		nams <- c()
		for (i in 1:length(vector1)) {
			vals <- c(vals, vector1[i], vector2[i])
			nams <- c(nams, names(vector1)[i], names(vector2)[i])
		}
		names(vals) <- nams
		return(vals)
	} else {
		stop("vectors do not match in length.")
	}
}

# Give column with estimate and standard errors
get_table_n <- function(ret, rv) {
	est <- fit_rGARCH_GAS_n(ret, rv)
	sd <- sqrt(diag(solve((est$output)$hessian)))
	coefs <- est[["optim_vals"]]
	names(sd) <- rep("", length(sd))
	table <- zip(coefs, sd)
	LogLiks <- unlist(LogLik_n(est$output$par, ret, log(rv), log(rv)[1], report = T))
	return(c(table, LogLiks))
}

get_table_jump_n <- function(ret, rv, jumps) {
	est <- fit_rGARCH_GAS_jumps_n(ret, rv, jumps)
	sd <- sqrt(diag(solve((est$output)$hessian)))
	coefs <- est[["optim_vals"]]
	names(sd) <- rep("", length(sd))
	table <- zip(coefs, sd)
	LogLiks <- unlist(LogLik_jumps_n(est$output$par, ret, log(rv), jumps, log(rv)[1], report = T))
	return(c(table, LogLiks))
}

get_table_multiple <- function(ret, rv) {
	est <- fit_rGARCH_multiple(ret, rv)
	sd <- sqrt(diag(solve((est$output)$hessian)))
	coefs <- (est$output)$par
	names(sd) <- rep("", length(sd))
	names(coefs) <- c(c("ω", "β", "d1", "d2", paste("γ", colnames(rv), sep="_")),c(sapply(colnames(rv), function(i) paste(i, c("ξ", "φ", paste("δ", 1:2, sep="_")), sep=":"))))

	return(zip(coefs, sd))
}

get_table_MLE_jump_n <- function(ret, rv, jumps) {
	est <- fit_rGARCH_MLE_jumps_n(ret, rv, jumps)
	sd <- sqrt(diag(solve((est$output)$hessian)))
	coefs <- est[["optim_vals"]]
	names(sd) <- rep("", length(sd))
	table <- zip(coefs, sd)
	LogLiks <- unlist(LogLik_MLE_jump_n(est$output$par, ret, log(rv), jumps, log(rv)[1], report = T))
	return(c(table, LogLiks))
}

get_table_MLE_n <- function(ret, rv) {
	est <- fit_rGARCH_MLE_n(ret, rv)
	sd <- sqrt(diag(solve((est$output)$hessian)))
	coefs <- est[["optim_vals"]]
	names(sd) <- rep("", length(sd))
	table <- zip(coefs, sd)
	LogLiks <- unlist(LogLik_MLE_n(est$output$par, ret, log(rv), log(rv)[1], report = T))
	return(c(table, LogLiks))
}

read_in_and_standardize_data <- function(which_file) {
	dates <- read.csv("../Data/new_data/dateFX.csv", header = F)
	data <- read.csv(file.path("..","Data","new_data",which_file), header = F)

	names(data) <- c("Logreturn", "RV5min", "BPV", "MedRV", "RK1min", "TSRV", "JWTSRV", "JVBPV", "JVMedRV", "JVJWTSRV",  "JWTSRV_W1", "JWTSRV_W2", "JWTSRV_W3", "JWTSRV_W4", "JWTSRV_W5")

	data <- xts(data, as.Date(strptime(apply(dates, 1, function(i) paste(i[3:1], collapse="-")), "%d-%m-%Y")))

	factor <- 1000
	# Normování returnů
	data[,"Logreturn"] <- data[,"Logreturn"]*factor - mean(data[,"Logreturn"]*factor)

	# Normování rv
	rv_indeces <- c("RV5min","RK1min","TSRV")
	for (i in rv_indeces) {
		data[,i] <- data[,i]*(factor^2)
	}

	data[,c("JWTSRV_W1", "JWTSRV_W2", "JWTSRV_W3", "JWTSRV_W4", "JWTSRV_W5")] <- data[,c("JWTSRV_W1", "JWTSRV_W2", "JWTSRV_W3", "JWTSRV_W4", "JWTSRV_W5")]*(factor^2)

	# Normování rv + jump
	data[,c("BPV", "JVBPV")] <- data[,c("BPV", "JVBPV")]*(factor^2)
	data[,c("MedRV","JVMedRV")] <- data[,c("MedRV","JVMedRV")]*(factor^2)
	data[,c("JWTSRV","JVJWTSRV")] <- data[,c("JWTSRV","JVJWTSRV")]*(factor^2)

	return(data)
}

forecastSimplified_rGARCH_GAS_n <- function(logrets, volatility, initialVals = NULL, steps = 1) {
	if (is.null(initialVals)) {
		fit <- fit_rGARCH_GAS_n(logrets, volatility)
	} else {
		fit <- fit_rGARCH_GAS_n(logrets, volatility, initialVals)
	}
	fitted <- recursive_filter_n(fit$output$par, logrets, log(volatility))
	forecasts <- forecast_rGARCH_GAS_n(fit, fitted, steps)
	if (length(steps)!=1) {
		out <- sapply(lapply(forecasts,exp), sum)
		out <- sapply(1:length(steps), function(i) sqrt(out[i]/steps[i]))
		names(out) <- sapply(steps, as.character)
		return(list(pred = out, fit = fit$output$par))
	} else {
		return(list(pred = exp(forecasts), fit = fit$output$par))
	}
}

forecastSimplified_rGARCH_GAS_jump_n <- function(logrets, volatility, jumps, initialVals = NULL, steps = 1, ar = 0, cummulated = TRUE) {
	if (is.null(initialVals)) {
		fit <- fit_rGARCH_GAS_jumps_n(logrets, volatility, jumps)	
	} else {
		fit <- fit_rGARCH_GAS_jumps_n(logrets, volatility, jumps, initialVals)	
	}
	fitted <- recursive_filter_jumps_n(fit$output$par, logrets, log(volatility), jumps)
	forecast <- forecast_rGARCH_GAS_jump_n(fit, fitted, jumps, ar = ar, steps = steps)
	if (length(steps)!=1) {
		if (cummulated) {
			out <- sapply(forecast, function(i) sum(exp(i$logvol) + fit$output$par["jumppar"] * i$jumps$pred))
			out <- sapply(1:length(steps), function(i) sqrt(out[i]/steps[i]))
			names(out) <- sapply(steps, as.character)
			return(list(pred = out, fit = fit$output$par))
		} else {
			return(list(pred = forecast, fit = fit$output$par))
		}
	} else {
		if (cummulated) {
			return(list(pred = sum(exp(forecast$logvol) + fit$output$par["jumppar"] * forecast$jumps$pred), fit = fit$output$par))
		} else {
			return(list(pred = forecast, fit = fit$output$par))
		}	
	}
}

forecastSimplified_rGARCH_MLE_jump_n <- function(logrets, volatility, jumps, initialVals = NULL, steps = 1, ar = 0, cummulated = TRUE) {
	if (is.null(initialVals)) {
		fit <- fit_rGARCH_MLE_jumps_n(logrets, volatility, jumps)	
	} else {
		fit <- fit_rGARCH_MLE_jumps_n(logrets, volatility, jumps, initialVals)
	}
	fitted <- recursive_filter_jumps_mle_n(fit$output$par, logrets, log(volatility), jumps)
	forecast <- forecast_rGARCH_MLE_jump_n(fit, fitted, jumps, log(volatility[length(volatility)]), ar = ar, steps = steps)
	if (length(steps)!=1) {
		if (cummulated) {
			out <- sapply(forecast, function(i) sum(exp(i$logvol) + fit$output$par["jumppar"] * i$jumps$pred))
			out <- sapply(1:length(steps), function(i) sqrt(out[i]/steps[i]))
			names(out) <- sapply(steps, as.character)
			return(list(pred = out, fit = fit$output$par))
		} else {
			return(list(pred = forecast, fit = fit$output$par))
		}
	} else {
		if (cummulated) {
			return(list(pred = sum(exp(forecast$logvol) + fit$output$par["jumppar"] * forecast$jumps$pred), fit = fit$output$par))
		} else {
			return(list(pred = forecast, fit = fit$output$par))
		}	
	}
}

forecastSimplified_rGARCH_MLE_n <- function(logrets, volatility, initialVals = NULL, steps = 1, ar = 0, cummulated = TRUE) {
	if (is.null(initialVals)) {
		fit <- fit_rGARCH_MLE_n(logrets, volatility)	
	} else {
		fit <- fit_rGARCH_MLE_n(logrets, volatility, initialVals)
	}
	fitted <- recursive_filter_mle_n(fit$output$par, logrets, log(volatility))
	forecast <- forecast_rGARCH_MLE_n(fit, fitted, log(volatility[length(volatility)]), ar = ar, steps = steps)
	if (length(steps)!=1) {
		out <- sapply(lapply(forecast,exp), sum)
		out <- sapply(1:length(steps), function(i) sqrt(out[i]/steps[i]))
		names(out) <- sapply(steps, as.character)
		return(list(pred = out, fit = fit$output$par))
	} else {
		return(list(pred = exp(forecast), fit = fit$output$par))
	}
}

forecastSimplified_GARCH_MLE_n <- function(logrets, steps = 1, ar = 0, cummulated = TRUE) {
	fit <- fit_GARCH_MLE_n(logrets)
	fitted <- recursive_filter_GARCH_n(fit$output$par, logrets)
	forecast <- forecast_GARCH_MLE_n(fit, fitted, steps = steps)
	if (length(steps)!=1) {
		return(list(pred = forecast, fit = fit$output$par))
		out <- sapply(forecast, sum)
		out <- sapply(1:length(steps), function(i) sqrt(out[i]/steps[i]))
		names(out) <- sapply(steps, as.character)
		return(list(pred = out, fit = fit$output$par))
	} else {
		return(list(pred = forecast, fit = fit$output$par))
	}
}

forecastSimplified_multiple <- function(logrets, volatility, initialVals = NULL, steps = 1, cummulated = TRUE) {
	if (is.null(initialVals)) {
		fit <- fit_rGARCH_multiple(logrets, volatility)	
	} else {
		fit <- fit_rGARCH_multiple(logrets, volatility, initialVals)
	}
	fitted <- recursive_filter_multi(fit$output$par, logrets, volatility)
	forecast <- forecast_rGARCH_multiple(fit, fitted, steps = steps)
	if (length(steps)!=1) {
		a <- exp(forecast)
		out <- sapply(steps, function(s) sqrt(sum(a[1:s]/s)))
		names(out) <- sapply(steps, as.character)
		return(list(pred = out, fit = fit$output$par))
	} else {
		return(list(pred = exp(forecast), fit = fit$output$par))
	}
}