LogLik_MLE_jump_n <- function(params, ret, logrv, jumps, h1, report = FALSE) {
	params <- c(params, h1)
	t <- recursive_filter_jumps_mle_n(params, ret, logrv, jumps)
	return(LogLik_MLE(t, report))
}

fit_rGARCH_MLE_jumps_n <- function(ret, rv, jumps, initialVals = c(xi = 0.0, phi = 1.0805, d1 = 0.0, d2 = 0.0, omega = 0, gam = 0.211, bet = 0.9, jumppar = 0)) {
	upper = c(xi = 10.0, phi = 1.5, d1 = 0.5, d2 = 0.5, omega = 1, gam = 0.95, bet = 0.9999, jumppar = 0.2)
	lower = c(xi = -10.0, phi = 0.5, d1 = -0.5, d2 = -0.5, omega = -1, gam = 0.001, bet = 0.3, jumppar = -0.2)
	
	c = optim(initialVals, LogLik_MLE_jump_n, ret = ret, logrv = log(rv), h1 = log(rv)[1], jumps = jumps, upper = upper, lower = lower, method = "L-BFGS-B", control = list(maxit = 1000, factr = 10), hessian = T)

	optim_params <- c$par
	names(optim_params) <- c("ξ", "φ", "d1", "d2", "ω", "γ", "β", "jumppar")
	return(list(optim_vals = optim_params, output = c))	
}

# fit_rGARCH_MLE_jumps_n <- function(ret, rv, jumps, initialVals = c(xi = 0.0, phi = 1.0805, d1 = 0.0, d2 = 0.0, delta_1 = 0.0, delta_2 = 0.0, omega = 0, gam = 0.211, bet = 0.9, jumppar = 0)) {
	
# 	c = optim(initialVals, LogLik_MLE_jump_n, ret = ret, logrv = log(rv), h1 = log(rv)[1], jumps = jumps, method = "L-BFGS-B", control = list(maxit = 1000, factr = 1e10), hessian = T)

# 	optim_params <- c$par
# 	names(optim_params) <- c("ξ", "φ", "d1", "d2", "δ_1", "δ_2", "ω", "γ", "β", "jumppar")
# 	return(list(optim_vals = optim_params, output = c))	
# }

LogLik_MLE <- function(filter, report = FALSE) {
	k <- ncol(filter[["u"]])
	T <- nrow(filter[["u"]])

	part1 <- 0.5*sum(log(2*pi) + filter[["lambda"]] + filter[["z"]]^2) 
	part2 <- T/2*(k + k*log(2*pi) + { r <- determinant(t(filter[["u"]])%*%filter[["u"]]/T, log=TRUE) ; if(r$sign <= 0) 1e10 else r$modulus })

	if (report) {
		return(list(u = -part2, rest = -part1))
	} else {
		if (((part1+part2)==Inf)||((part1+part2)==-Inf)||(is.nan((part1+part2)))) {
			return(1e10)
		} else {
			return(part1 + part2)
		}
	}
}